/** @addtogroup STM32L4xx_System_Private_Includes
  * @{
  */

#include "stm32l432xx.h"
#include "sys_l4.h"

/** @addtogroup STM32L4xx_System_Private_Defines
  * @{
  */
#define MSI_100K     0x000
#define MSI_200K     0x010
#define MSI_400K     0x020
#define MSI_800K     0x030
#define MSI_1M       0x040
#define MSI_2M       0x050
#define MSI_4M       0x060
#define MSI_8M       0x070
#define MSI_16M      0x080
#define MSI_24M      0x090
#define MSI_32M      0x0A0
#define MSI_48M      0x0B0

/************************* Miscellaneous Configuration ************************/
/*!< Uncomment the following line if you need to relocate your vector Table in
     Internal SRAM. */
/* #define VECT_TAB_SRAM */
#define VECT_TAB_OFFSET  0x00 /*!< Vector Table base offset field.
                                   This value must be a multiple of 0x200. */
/******************************************************************************/
#define FOC_CLK 80  
static const float Time_us_Base=FOC_CLK/8.0f;
static const u32 Time_ms_Base=FOC_CLK*125;

void ysu(unsigned int us)
{
	if(us)
	{
		SysTick->LOAD=Time_us_Base*us;
		SysTick->VAL=0;
		SysTick->CTRL=1;
		while(!(0x10000&SysTick->CTRL));
		SysTick->CTRL=0;
	}
}


void ysm(unsigned int ms)
{
  u32 kk;
  kk=ms*Time_ms_Base;
  while(kk)
  {
    if(kk>16000000) 
    {
      SysTick->LOAD=16000000;
      kk-=16000000;
    }
    else 
    {
      SysTick->LOAD=kk;
      kk=0;
    }
    SysTick->VAL=0;
    SysTick->CTRL=1;
    while(!(0x10000&SysTick->CTRL));
    SysTick->CTRL=0;
  }
}

void IOset(GPIO_TypeDef *IOx,u32 Tbits,u32 Obit,u32 moder,u32 ospeedr,u32 pupdr,u32 otyper,u32 lckr)
{
  u32 temp;
  temp=(u32)~(Tbits*3);
  IOx->MODER&=temp;
  IOx->MODER|=Tbits*moder;
  IOx->OSPEEDR&=temp;
  IOx->OSPEEDR|=Tbits*ospeedr;
  IOx->PUPDR&=temp;
  IOx->PUPDR|=Tbits*pupdr;
  if(otyper)
  {
    IOx->OTYPER|=Obit;
  }
  else
  {
    IOx->OTYPER&=~Obit;
  }
  if(lckr)
  {
    Obit=IOx->LCKR|Obit;
    Obit&=0x0000ffff;
    IOx->LCKR=0x10000|Obit;
    IOx->LCKR=Obit;
    IOx->LCKR=0x10000|Obit;
    Obit=IOx->LCKR;
  }
}

void MSI_To_PLL(void)
{
  FLASH->ACR=0x1800;
  
  RCC->CR |= RCC_CR_MSION;
  while(!(RCC->CR&0x02));
  
  RCC->CFGR = 0x00000000;
  FLASH->ACR=0;
  FLASH->ACR=0x700;
  FLASH->ACR|=0x04;
  /*                        PLL_N       PLL_M  PLLSRC  */
  RCC->PLLCFGR =(uint32_t)(FOC_CLK<<8 | 1<<4 | 0x01);
  RCC->PLLCFGR|=(uint32_t)0x01000000;
  /* Reset HSEON, CSSON , HSION, and PLLON bits */
  RCC->CR &= (uint32_t)0xEAF6FFFF;
  
  RCC->CR |= (uint32_t)0x01000000;   //PLLON
  while(!(RCC->CR&0x02000000));
  
  RCC->CFGR |=(uint32_t)0x03;        //PLL selected as system clock
  while(((RCC->CFGR&0x0C)!=0x0C));
}

void USE_MSI(void)
{
  FLASH->ACR=0x1800;
  RCC->CR |= RCC_CR_MSION;
  while(!(RCC->CR&0x02));
  FLASH->ACR=0;
  FLASH->ACR=0x700;
  FLASH->ACR|=0x03;
  RCC->CR &= (uint32_t)0xffffff0f;
  RCC->CR |=(uint32_t) MSI_48M;
  RCC->CSR|=(uint32_t) 0x0700;
  RCC->CR|=(uint32_t)0x08;
  RCC->CFGR = 0x00000000;
}

void HSI_To_PLL(void)
{
  FLASH->ACR=0x1800;
  
  RCC->CR &= (uint32_t)0xFAF6FFFF;   
  RCC->CR |= RCC_CR_HSION;          //16MHZ
  while(!(RCC->CR&RCC_CR_HSIRDY));
  
  RCC->CFGR = 0x00000000;
  FLASH->ACR=0;
  FLASH->ACR=0x700;
  FLASH->ACR|=0x04;
  /*                        PLL_N*80       PLL_M/8  PLLSRC  */
  RCC->PLLCFGR =(uint32_t)(FOC_CLK<<8 | 7<<4 | 0x02);
  RCC->PLLCFGR|=(uint32_t)0x01000000;    //PLL OUTPUT
  
  RCC->CR |= (uint32_t)0x01000000;   //PLLON
  while(!(RCC->CR&0x02000000));
  
  RCC->CFGR |=(uint32_t)0x03;        //PLL selected as system clock
  while(((RCC->CFGR&0x0C)!=0x0C));
}

void HSE_To_PLL(void)
{
  RCC->CR &= (uint32_t)0xEAF6FFFF;
  RCC->CR |= RCC_CR_HSEON;          //8MHZ
  while(!(RCC->CR&RCC_CR_HSERDY));
  
  RCC->CFGR = 0x00000000;
  FLASH->ACR=0;
  FLASH->ACR=0x700;
  FLASH->ACR|=0x04;
  /*                        PLL_N       PLL_M  PLLSRC  */
  RCC->PLLCFGR =(uint32_t)(FOC_CLK<<8 | 3<<4 | 0x03);
  RCC->PLLCFGR|=(uint32_t)0x01000000;
  
  RCC->CR |= (uint32_t)0x01000000;   //PLLON
  while(!(RCC->CR&0x02000000));
  
  RCC->CFGR |=(uint32_t)0x03;        //PLL selected as system clock
  while(((RCC->CFGR&0x0C)!=0x0C));
}

void SystemInit(void)
{
  SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 Full Access */
//  USE_MSI();
//  MSI_To_PLL();
  HSI_To_PLL();
//  HSE_To_PLL();
  /* Disable all interrupts */
  RCC->CIER = 0x00000000;

#ifdef VECT_TAB_SRAM
  SCB->VTOR = SRAM_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal SRAM */
#else
  SCB->VTOR = FLASH_BASE | VECT_TAB_OFFSET; /* Vector Table Relocation in Internal FLASH */
#endif
}

